/*
 * @LastEditors: Please set LastEditors
 * @LastEditTime: 2022-04-28 10:25:38
 */
import axios from 'axios';
// import UtilVar from "@/config/UtilVar";
import router from '@/router'
import {getAdminId, getToken} from "@/utils/auth";
import {tansParams} from "@/utils/ruoyi";
import cache from "@/plugins/cache";
import errorCode from "@/utils/errorCode";
import {Message, MessageBox} from "element-ui";
import store from "@/store";
import {isRelogin} from "@/api/utils/request";
// import cache from "@/plugins/cache";

let baseUrl = process.env.VUE_APP_BASE_API
const CancelToken = axios.CancelToken;
export {baseUrl};
// axios.defaults.withCredentials = true;


//5/27 修改
axios.defaults.headers["Content-Type"] = "application/json;charset=utf-8";
// 对应国际化资源文件后缀
axios.defaults.headers["Content-Language"] = "zh_CN";
// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: process.env.VUE_APP_BASE_API,
  // 超时
  timeout: 10000,
});

// request拦截器
service.interceptors.request.use(
  (config) => {
    // 是否需要设置 token
    const isToken = (config.headers || {}).isToken === false;
    // 是否需要防止数据重复提交
    const isRepeatSubmit = (config.headers || {}).repeatSubmit === false;
    if (getToken() && !isToken) {
      // config.headers['ADMIN_TOKEN'] = getToken() // 让每个请求携带自定义token 请根据实际情况自行修改
      config.headers = {
        CLIENT_TYPE: "locWeb",
        ADMIN_ID: getAdminId(),
        ADMIN_TOKEN: getToken(),
      };
    }
    // get请求映射params参数
    if (config.method === "get" && config.params) {
      let url = config.url + "?" + tansParams(config.params);
      url = url.slice(0, -1);
      config.params = {};
      config.url = url;
    }
    if (
      !isRepeatSubmit &&
      (config.method === "post" || config.method === "put")
    ) {
      const requestObj = {
        url: config.url,
        data:
          typeof config.data === "object"
            ? JSON.stringify(config.data)
            : config.data,
        time: new Date().getTime(),
      };
      const sessionObj = cache.session.getJSON("sessionObj");
      if (
        sessionObj === undefined ||
        sessionObj === null ||
        sessionObj === ""
      ) {
        cache.session.setJSON("sessionObj", requestObj);
      } else {
        const s_url = sessionObj.url; // 请求地址
        const s_data = sessionObj.data; // 请求数据
        const s_time = sessionObj.time; // 请求时间
        const interval = 1000; // 间隔时间(ms)，小于此时间视为重复提交
        // if (
        //   s_data === requestObj.data &&
        //   requestObj.time - s_time < interval &&
        //   s_url === requestObj.url
        // ) {
        //   const message = "数据正在处理，请勿重复提交";
        //   console.warn(`[${s_url}]: ` + message);
        //   return Promise.reject(new Error(message));
        // } else {
        //   cache.session.setJSON("sessionObj", requestObj);
        // }
        cache.session.setJSON("sessionObj", requestObj);
      }
    }
    return config;
  },
  (error) => {
    console.log("2", error);
    Promise.reject(error);
  }
);

// 添加请求拦截器
// axios.interceptors.request.use(function (config) {
//     // 在发送请求之前做些什么 传token
//     let token = localStorage.getItem("token");
//     config.headers.common['Content-Type'] = "application/json;charset=utf-8";
//     config.headers.common['token'] = token;  //Authorization
//     return config;
// }, function (error) {
//     // 对请求错误做些什么
//     console.log(error)
//     return Promise.reject(error);
// });
/**
 * @响应拦截
 */
// 响应拦截器
service.interceptors.response.use(
  (res) => {
    // 未设置状态码则默认成功状态
    const code = res.data.code || 200;
    // 获取错误信息
    const msg = errorCode[code] || res.data.message;
    // 二进制数据则直接返回
    if (
      res.request.responseType === "blob" ||
      res.request.responseType === "arraybuffer"
    ) {
      return res.data;
    }
    if (code === 2) {
      if (!isRelogin.show) {
        isRelogin.show = true;
        MessageBox.confirm(
          "登录状态已过期，您可以继续留在该页面，或者重新登录",
          "系统提示",
          {
            confirmButtonText: "重新登录",
            cancelButtonText: "取消",
            type: "warning",
          }
        )
          .then(() => {
            isRelogin.show = false;
            store.dispatch("LogOut").then(() => {
              location.href = process.env.VUE_APP_CONTEXT_PATH;
            });
          })
          .catch(() => {
            isRelogin.show = false;
          });
      }
      return Promise.reject("无效的会话，或者会话已过期，请重新登录。");
    } else {
      return res.data;
    }
  },

  /**
   * else if (code === 500) {
      Message({
        message: msg,
        type: 'error'
      })
      return Promise.reject(new Error(msg))
    }

   else if (code !== 20000) {
      Notification.error({
        title: msg
      })
      return Promise.reject('error')
    }
   */
  (error) => {
    console.log("err" + error);
    let {message} = error;
    if (message == "Network Error") {
      message = "后端接口连接异常";
    } else if (message.includes("timeout")) {
      message = "系统接口请求超时";
    } else if (message.includes("Request failed with status code")) {
      message = "系统接口" + message.substr(message.length - 3) + "异常";
    }
    Message({
      message: message,
      type: "error",
      duration: 5 * 1000,
    });
    return Promise.reject(error);
  }
);

// axios.interceptors.response.use(response => {
//     if (response.status !== 20000) {
//         return Promise.reject(response)
//     }
//       /**
//      * @code 登录过期 token验证失败 根据后端调
//     //  */
//     // if (response.data.code == UtilVar.code) {
//     //     // router.push("/login")
//     // }
//     return response.data
// }, error => {
//     // console.log('axiosError',error);
//     let err = {
//         success: false,
//         msg: "token过期未知异常，请联系管理员！"
//     }
//     return Promise.reject(err)
// })

// let configs_ENC = {
//     headers: { 'enc': UtilVar.ENC }
// }
//处理是否加密数据
let isEncryptionParam = (params) => {
  return params

}
export const GET = async (url, params) => {
  try {
    params = isEncryptionParam(params)
    const data = await axios.get(`${baseUrl}${url}`, {
      params: params,
      headers: false,
    }, false);
    return data;
  } catch (error) {
    return error;
  }
}
//没有基地址 访问根目录下文件
// export function GETNOBASE(url,query) {
//   return axios({
//     url:url,
//     method: "get",
//     params: query
//   });
// }
export const GETNOBASE = async (url, params) => {
  try {
    const data = await axios.get(url, {
      params: params,
    });
    return data;
  } catch (error) {
    return error;
  }
}
export const POST = async (url, params) => {
  try {
    params = isEncryptionParam(params)
    const data = await axios.post(`${baseUrl}${url}`, params, false);
    return data;
  } catch (error) {
    return error;
  }
}
export const PUT = async (url, params) => {
  try {
    params = isEncryptionParam(params)
    const data = await axios.put(`${baseUrl}${url}`, params, false);
    return data;
  } catch (error) {
    return error;
  }
}
export const DELETE = async (url, params) => {
  // console.log(params)
  try {
    params = isEncryptionParam(params)
    const data = await axios.delete(`${baseUrl}${url}`, {data: params, headers: configs_ENC.headers}, configs_ENC);
    return data;
  } catch (error) {
    return error;
  }
}


/**
 * @文件类型提交方法
 */
let configs = {
  headers: {'Content-Type': 'multipart/form-data'},

}
export const FILESubmit = async (url, params, config) => {
  try {
    const data = await axios.post(`${baseUrl}${url}`, params, {
      ...configs,
      cancelToken: new CancelToken(function executor(c) {
        config.setCancel && config.setCancel(c)
      }),
      onUploadProgress: (e) => {
        if (e.total > 0) {
          e.percent = e.loaded / e.total * 100;
        }
        // console.log(config)
        config.onProgress && config.onProgress(e)
      },

    });
    return data;
  } catch (err) {
    return err;
  }
}

/**
 * 下载文档流
 * @param {config.responseType} 下载文件流根据后端 配置   arraybuffer || blod
 */
export const FILE = async (config = {}, body, params) => {
  try {
    const data = await axios({
      method: config.method || 'get',
      url: `${baseUrl}${config.url}`,
      data: body,
      params: params,
      responseType: config.responseType || 'blob',
      onDownloadProgress: (e) => {
        // console.log(e,e.currentTarget)
        // if (e.currentTarget.response.size > 0) {
        //     e.percent = e.loaded / e.currentTarget.response.size * 100;
        // }
        // event.srcElement.getResponseHeader('content-length')
        config.onProgress && config.onProgress(e)
      },
    });
    return data;
  } catch (err) {
    return err;
  }
}


